// Definitions

/// {@template state_transition_entry}
/// Entry for [StateGraph], contains optional [transition] and [sideEffect].
/// {@endtemplate}
class StateTransitionEntry<State, Event> {
  /// State to new state conversion.
  final StateTransition<State, Event> transition;

  /// Side effect of transition (e.g. logging).
  final SideEffect<State, Event> sideEffect;

  /// {@macro state_transition_entry}
  StateTransitionEntry({this.transition, this.sideEffect});
}

/// Transition builder function.
typedef State StateTransition<State, Event>(State state, Event event);

/// Side effect builder function.
typedef void SideEffect<State, Event>(State state, Event event);

// Sugar functions

/// Makes [StateTransitionEntry] with just state-to-state [transition].
StateTransitionEntry<State, Event> transition<State, Event>(
  StateTransition<State, Event> transition,
) {
  return StateTransitionEntry<State, Event>(transition: transition);
}

/// Makes [StateTransitionEntry] with both [transition] and [sideEffect].
StateTransitionEntry<State, Event> transitionWithSideEffect<State, Event>(
  StateTransition<State, Event> transition,
  SideEffect<State, Event> sideEffect,
) {
  return StateTransitionEntry<State, Event>(
    transition: transition,
    sideEffect: sideEffect,
  );
}

/// Makes [StateTransitionEntry] with just [sideEffect] of transition.
StateTransitionEntry<State, Event> sideEffect<State, Event>(
  SideEffect<State, Event> action,
) {
  return StateTransitionEntry<State, Event>(sideEffect: action);
}
